unit ASMSample;
{
  ASMSample
  
  Demonstation of const and var parameters in delphi together with 
  assembly language.

    ---------------------
     written by socke2ooo
     ----------------------

  I found many questions about this in newsgroups so I decided to 
  write this little sample.

  const parameters are passed "by value" (that means the content of
  the parameter is passed)
  var parameters are passed "by pointer" (that means that not the content
  but a pointer to it is passed. that is a bit difficult to handle but
  enables you to modify the original values.)

  Delphi passes the parameters (values or pointers) in the following way:
    o 1st parameter is passed in EAX
    o 2nd parameter is passed in EDX
    o 3rd parameter is passed in ECX
    o the last parameter is passed in [ebp+8]
    o the last but one parameter is passed in [ebp+12]
    o the last but two parameter is passed in [ebp+16]
    o and so on...
  Important is that you save and restore the EBX register. Modifications 
  would cause Delphi to hang or give ugly access violations.

  The issue of this sample is to handle expiration data. The admin should be able
  to specify:
    a) Account expiration date
        -and/or-
    b) A number of logins after the account expires.
  The Data is stored in a DWord.

  Feel free to send comments to: socke-99@gmx.de

}

interface


uses Windows;

type
  TExpData = DWord;

function EncodeExpData(
  const UseRemLogins: LongBool; // Must use LongBool for some reason I don't know
  const RemLogins, Year, Month, Day: Integer): TExpData;

procedure DecodeExpData(
  const Data: TExpData;
  var UseRemLogins: LongBool;
  var RemLogins, Year, Month, Day: Integer);


implementation

function EncodeExpData(
  const UseRemLogins: LongBool; // Must use LongBool for some reason I don't know
  const RemLogins, Year, Month, Day: Integer): TExpData;
{
  This procedure places the parameters into one DWord:
  .----------------------------------++--------------------------------.
  |               MSW                ||              LSW               |
  |--.----------------------.--------++----------.---------------------|
  |* |  remaining logins    |   Day  ||  Month   |        Year         |
  |--+----------------------+--------++----------+---------------------|
  |31|30                  21|20    16||15      12|11                  0|
  '--'----------------------'--------''----------'---------------------'
  *) UseRemLogins

  On start is:
    EAX         UseRemLogins      ok
    ECX         Year              ok
    EDX         RemLogins
    SS:[ESP+8]  Day
    SS:[ESP+12] Month

  At return EAX holds the result
}
asm

  { we need to save some registers at first
      (if we won't do so delphi would give us some access violations) }
  PUSHF
  PUSH    EBX

  { if RemainingLogins > 1023 set RemaingLogins to 1023 }
  CMP     EDX, 1023
  JBE     @@1
  MOV     EDX, 1023
@@1:

  { copy the UseRemLogins to the first bit of EAX }
  XOR     EBX,EBX     { set EBX to 0 }
  CMP     EAX,0
  SETNE   BL          { set BL to 1 if UseRemainingLogins is True (<>0) }
  PUSH    EBX         { copy EBX... }
  POP     EAX         {    ... to EAX }

  SHL     EAX,31      { move the set bit to bit 31 }
  AND     ECX,0FFFh   { remember, ECX is Years parameter until we change it }
  OR      EAX,ECX     { copy the lowest 3 halfbytes (rem.: we masked) of ECX into EAX }
  SHL     EDX,21      { Shift the the RemainingLogins parameter to the right position }
  AND     EDX,7FE00000h { Mask the unwanted part out }
  OR      EAX,EDX     { copy the remaining bits into the result (EAX) }
  MOV     EBX,Month   { EBX holds now the Month parameter }
  MOV     EDX,Day     { EDX holds now the Day parameter }

  SHL     EBX,12      { Shift EBX (now the month) to the right position to merge }
  SHL     EDX,16      { Shift EDX (now the day) to the right position }
  AND     EBX,0F000h  { mask out the unwanted bits }
  AND     EDX,1F0000h {      "           "         }
  OR      EAX,EBX     { merge result with the month }
  OR      EAX,EDX     { merge result with the day }

  { finally we  want to restore the saved registers }
  POP     EBX
  POPF
{
  now we did it. If you have questions or suggestions to optimize this code
  feel free to mail me at: socke-99@gmx.de
  But: I am not here to teach you assembly language!
}

end;



procedure DecodeExpData(
  const Data: TExpData;
  var UseRemLogins: LongBool;
  var RemLogins, Year, Month, Day: Integer);
{ it the EncodeExpData function is warp 10 this is warp 8 }
asm
  { save registers }
  PUSH EBX

  AND     dword ptr[EDX], 0   { UseRemLogins := 0 }
  BT      EAX,31              { if bit 31 of Data is set then ... }
  SETC    byte ptr[EDX]       { ... UseRemLogins := True else UseRemLogins := False }

  { figure out RemLogins }
  PUSH    EAX                 { copy EAX ... }
  POP     EBX                 {  ... to EBX }
  AND     EBX,7FE00000h       { mask out all bits not belonging to RemLogins }
  SHR     EBX,21              { Shift EBX to the right position }
  PUSH    EBX                 { copy EBX ... }
  POP     dword ptr[ECX]      {   ... to RemLogins }
//now all X-registers except EAX are free to juggle with
  PUSH    EAX
  POP     EBX
  AND     EBX,0FFFh           { extract bits for Day }
  PUSH    dword ptr [ebp+16]
  POP     ECX
  MOV     dword ptr[ECX],EBX  { Day := EBX }

  PUSH    EAX
  POP     EBX
  AND     EBX,0F000h          { extract bits for Month }
  SHR     EBX,12              { shift EBX into right position }
  PUSH    dword ptr [ebp+12]
  POP     ECX
  MOV     dword ptr[ECX],EBX  { Month := EBX }

  AND     EAX,1F0000h         { extract bits for Day }
  SHR     EAX,16              { shift EAX into right position }
  PUSH    dword ptr [ebp+8]
  POP     ECX
  MOV     dword ptr[ECX],EAX  { Day := EAX }

  { restore registers }
  POP EBX

end;

end.
